<?php
/*
 * Plugin Name: MYPAYGA Payment Gateway
 * Plugin URI: https://mypayga.com/plugins/woocommerce/payment-gateway-plugin.html
 * Description: Take Mobile money and credit card payments on your store.
 * Author: Minerve-Tech
 * Author URI: https://minerve-tech.com/
 * Version: 1.0
*/

/** This action hook registers our PHP class as a WooCommerce payment gateway */

    add_filter( 'woocommerce_payment_gateways', 'mypayga_add_gateway_class' );
    function mypayga_add_gateway_class( $gateways ) {
        $gateways[] = 'WC_Mypayga_Gateway'; // your class name is here
        return $gateways;
    }

    add_action('wp_enqueue_scripts','mypayga_events_3001');
        function mypayga_events_3001() {
            //wp_enqueue_script( 'mpgtoken', 'https://www.mypayga.com/mpgtoken.js');
    }

    add_filter( 'woocommerce_get_checkout_order_received_url', 'mpg_received_url_3001', 10, 2 );
    function mpg_received_url_3001($order_received_url, $that ){

        return $order_received_url;
    }

    add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'mpga_settings_link' );
    
    function mpga_settings_link( array $links ) {
        $url = get_admin_url() . "admin.php?page=wc-settings&tab=checkout&section=mypayga";
        $settings_link = '<a href="' . $url . '">' . __('Réglages', 'textdomain') . '</a>';
          $links[] = $settings_link;
        return $links;
    }

   function myapygaCreateTable3001() {

      global $wpdb;
      $charset_collate = $wpdb->get_charset_collate();
      $table_name = $wpdb->prefix.'mypayga_events';
      $sql = "CREATE TABLE $table_name (
            id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
            payment_token VARCHAR(200) NULL,
            uniqueID VARCHAR(30) NOT NULL,
            callback VARCHAR(10),
            status VARCHAR(10),
            reg_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            )";
      if ($wpdb->get_var("SHOW TABLES LIKE ".$table_name) != $table_name) {
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
      }
    }

register_activation_hook( __FILE__, 'myapygaCreateTable3001' );

/** The class itself, please note that it is inside plugins_loaded action hook */

add_action( 'plugins_loaded', 'mypayga_init_gateway_class' );
function mypayga_init_gateway_class() {

    class WC_Mypayga_Gateway extends WC_Payment_Gateway {

        /**
         * Class constructor, more about it in Step 3
        */

        public function __construct() {

            $this->id = 'mypayga'; // payment gateway plugin ID
            $this->icon = plugins_url('/assets/images/all-mypayga', __FILE__);
            $this->has_fields = false; // in case you need a custom credit card/Mobile form
            $this->method_title = 'Mypayga Gateway';
            $this->method_description = 'Acceptez les paiements par Carte bancaire et Mobile Money sur votre site.'; // will be displayed on the options page

            // gateways can support subscriptions, refunds, saved payment methods,
            // but in this tutorial we begin with simple payments
            $this->supports = array(
                'products'
            );

            // Method with all the options fields
            $this->init_form_fields();

            // Load the settings.
            $this->init_settings();
            $this->title = $this->get_option( 'title' );
            $this->description = $this->get_option( 'description' );
            $this->enabled = $this->get_option( 'enabled' );
            $this->testmode = 'yes' === $this->get_option( 'testmode' );
            $this->private_key = $this->testmode ? $this->get_option( 'test_private_key' ) : $this->get_option( 'private_key' );
            $this->publishable_key = $this->testmode ? $this->get_option( 'test_publishable_key' ) : $this->get_option( 'publishable_key' );

            // This action hook saves the settings
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

            // We need custom JavaScript to obtain check payment Event
            add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );
            
            // You can also register a webhook here
            add_action( 'woocommerce_api_mypayga_gateway', array( $this, 'check_ipn_response' ) );


     }

        /*** Plugin options, we deal with it in Step 3 too */
        
        public function init_form_fields(){

         $this->form_fields = array(
            'enabled' => array(
                'title'       => 'Enable/Disable',
                'label'       => 'Enable MYPAYGA Gateway',
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'MYPAYGA',
            ),
            'title' => array(
                'title'       => 'Title',
                'type'        => 'text',
                'description' => 'This controls the title which the user sees during checkout.',
                'default'     => 'MYPAYGA',
                'desc_tip'    => true,
            ),
            'description' => array(
                'title'       => 'Description',
                'type'        => 'textarea',
                'description' => 'This controls the description which the user sees during checkout.',
                'default'     => 'Pay with your credit or Mobile Money card via our payment gateway.',
            ),
            'testmode' => array(
                'title'       => 'Test mode',
                'label'       => 'Enable Test Mode',
                'type'        => 'checkbox',
                'description' => 'Place the payment gateway in test mode using test API keys.',
                'default'     => 'yes',
                'desc_tip'    => true,
            ),
            'test_publishable_key' => array(
                'title'       => 'Test Publishable Key',
                'type'        => 'text'
            ),
            'test_private_key' => array(
                'title'       => 'Test Private Key',
                'type'        => 'password',
            ),
            'publishable_key' => array(
                'title'       => 'Live Publishable Key',
                'type'        => 'text'
            ),
            'private_key' => array(
                'title'       => 'Live Private Key',
                'type'        => 'password'
            )
        );
    
    }

      
        /**
         * You will need it if you want your custom credit card form, Step 4 is about it
         */
        public function payment_fields() {

            /* ok, let's display some description before the payment form*/
            if ( $this->description ) {

                if ( $this->testmode ) {
                    $this->description .= '<br>TEST MODE ACTIVÉ. <br>En mode test, vous pouvez utiliser les numéros de cartes et téléphones répertoriés dans la <a href="https://www.mypayga.com/docs-page.html#section-2">documentation</a>.';
                    $this->description  = trim( $this->description );
                }

                echo wpautop( wp_kses_post( $this->description ) );
            }
         
            
        }

        /*
         * Custom CSS and JS, in most cases required only when you decided to go with a custom credit card form
        */

        public function payment_scripts() {

             // we need JavaScript to process a token only on cart/checkout pages, right?
            if (! is_checkout()) {
                return;
            }

            // and this is our custom JS in your plugin directory that works with token.js
            wp_register_script( 'js', plugins_url( 'mypayga-event-js.js', __FILE__ ), array( 'jquery', 'mypayga-event-js'));

            // in most payment processors you have to use PUBLIC KEY to obtain a token
            wp_localize_script( 'js', 'mypayga_params', array(
                'publishableKey' => $this->publishable_key
            ) );

            wp_enqueue_script( 'js' );
         
    
        }

        /*
         * Fields validation, more in Step 5
         */
        public function validate_fields() {

         if( empty( $_POST[ 'billing_first_name' ]) ) {
                wc_add_notice(  'First name is required!', 'error' );
                return false;
            }
            return true;

        }

        /*
         * We're processing the payments here
         */
        public function process_payment( $order_id ) {

            global $woocommerce;
            global $wpdb;
            global $wp_session;

            $mypayga_events = $wpdb->prefix.'mypayga_events';

            // we need it to get any order detailes
            $order = wc_get_order( $order_id );
            // Get Order ID and Key
            $uniqueID=$order->get_id();
            $WcorderKey=$order->get_order_key();
             
            /*
            * Array with parameters for API interaction
            */
            $sanitized_client_email = sanitize_email($order->get_billing_email());


            $ClientPhoneInp = $order->get_billing_phone();

            if(preg_match("/^[0-9]{9}$/", $ClientPhoneInp)) {
              $ClientPhone=$ClientPhoneInp;
            } else{
                wc_add_notice(  'Inavlid phone number.', 'error' );
                return;
            }

            $args = array(
                "email" => $sanitized_client_email,
                "firstname" =>sanitize_text_field( $order->get_billing_first_name() ),
                "lastname" => sanitize_text_field( $order->get_billing_last_name() ),
                "client_phone"=>$ClientPhone,
                "country"=>sanitize_text_field( $order->get_billing_country() ),
                "apikey" => sanitize_text_field( $this->private_key ),
                /* Not user inputs */
                "unique_id" => $order->get_id(),
                "amount" => $order->get_total(),
                "description"=>'WP- Payment',
                "order_key"=>sanitize_text_field( $order->get_order_key()),
                "order_source"=>'woocommerce',
                'checkout_order_received_url'=>esc_url( $order->get_checkout_order_received_url() ),/*Success url */
                'wc_get_checkout_url'=>esc_url( wc_get_checkout_url() ),
                'website_url'=>esc_url( get_site_url() ),
                'callback_url'=>esc_url( get_site_url() ).'/wc-api/mypayga_gateway',
                'fail_url'=>esc_url( wc_get_checkout_url() ),
                'success_url'=>esc_url( $order->get_checkout_order_received_url() ),
                'currency'=>get_woocommerce_currency()
                /* Not user inputs */
            );


        /*
         * API interaction with wp_remote_post()
         */
     
        $response = wp_remote_post('https://api.mypayga.com/v1/wordppress/create-order', array(
            'method' => 'POST',
            'sslverify' => true,
            'body' => $args)
        );

         if( !is_wp_error( $response ) ) {
     
            $body = json_decode( $response['body']);
            $message=$body->message;
     
             if ( $body->request_status == '200' ) {
                
               $payment_token=$body->wp_payment_uid;
               $checkout_url=$body->checkout_url;
               
               /* Add Data to custom events */

                $dataEvents=array(
                    'payment_token' => $payment_token, 
                    'uniqueID' => $order->get_id(), 
                    'status' => 400, 
                    'callback' => 0);

                $wpdb->insert($wpdb->prefix.'mypayga_events', $dataEvents);

                return array(
                        'result' => 'success',
                        'redirect' => $checkout_url
                );
     
             } else {
                wc_add_notice('Error, Please try again: '.$message, 'error' );
                return;
            }
     
        } else {
            wc_add_notice(  'Connection error.', 'error' );
            return;
        }         
                    
    }

        /*
         *  We need a check_ipn_response
         */
        public function check_ipn_response() {
        global $wpdb;

        $http_contents  = file_get_contents("php://input");
        
        $json = json_decode($http_contents, true);

        $OrderStatus=$json['OrderStatus'];
        $OrderStatus=$json['order_status'];
        $unique_id=$json['unique_id'];
        $message=$json['message'];
        $payment_token=$json['payment_token'];

        /* Callback update */

        $mypayga_events = $wpdb->prefix.'mypayga_events';

        $wpdb->query($wpdb->prepare("UPDATE 
            $mypayga_events SET callback='1',
            status='$OrderStatus'
        WHERE uniqueID=$unique_id"));

        $order = wc_get_order($unique_id);

            if ($OrderStatus == '200') {
                $order->payment_complete();
                $order->reduce_order_stock();
                $woocommerce->cart->empty_cart();

            } else {

            return;
            }               
        }
    }
}